###### This file replicates the analysis of happiness, age and income published in 
###### Toshkov (2021) 'The Relationship Between Age and Happiness Varies by Income', Journal of Happiness Studies
###### Part 3 of 6 (2D and 3D density plots). Last updated 1 September 2021

# Libraries and functions -------------------------------------------------
library(plot3D)
library(rgl)
source ('ggplot2 graphical settings.R')
# Prepare data -------------------------------------------------
x = dat9$age
y = dat9$income
z = dat9$happy

elevation.df = data.frame(x,y,z)
elevation.loess = loess(z ~ x*y, data = elevation.df, degree=2, span=0.4)
elevation.loess2 = mgcv::gam(z ~ s(x) + s(y) + ti (x, y), data=elevation.df) # the one used
elevation.loess3 = mgcv::gam(z ~ s(x, bs='cs') + s(y) + ti (x, y), data=elevation.df)

xmin<- min(x, na.rm = T)
xmax<- max(x, na.rm = T)
ymin<- min(y, na.rm = T)
ymax<- max(y, na.rm = T)

elevation.fit = expand.grid(list(x = seq(xmin, xmax, 1), y = seq(ymin, ymax, 1)))
z = predict(elevation.loess, newdata = elevation.fit)
z2 = predict(elevation.loess2, newdata = elevation.fit)
z3 = predict(elevation.loess3, newdata = elevation.fit)
elevation.fit$Height = as.numeric(z)
elevation.fit$Height2 = as.numeric(z2)
elevation.fit$Height3 = as.numeric(z3)

#prepare matrix with x and y and rows/cols and z as the cell values
u1<-unique(elevation.fit$x)
u2<-unique(elevation.fit$y)
temp.data<-matrix(NA, nrow=length(u1), ncol=length(u2))


for (i in 1:length(u1)){
  for (j in 1:length(u2)){
    temp.data[i,j]<-elevation.fit$Height2[elevation.fit$x==u1[i] & elevation.fit$y==u2[j]]
  }
}

#prepare matrix with equal rows and columns for plotly
elevation.fit2 = expand.grid(list(x = seq(xmin, xmax, 1), y = seq(ymin, ymax, 0.12)))
z2 = predict(elevation.loess2, newdata = elevation.fit2)
elevation.fit2$Height2 = as.numeric(z2)
u11<-unique(elevation.fit2$x)
u12<-unique(elevation.fit2$y)

temp.data2<-matrix(NA, nrow=length(u11), ncol=length(u12))
for (i in 1:length(u11)){
  for (j in 1:length(u12)){
    temp.data2[i,j]<-elevation.fit$Height2[elevation.fit2$x==u11[i] & elevation.fit2$y==u12[j]]
  }
}

save (temp.data, file = './data/3d-data.Rdata')
save (temp.data2, file = './data/3d-data2.Rdata')
save (elevation.fit, file = './data/3d-elevation-data.Rdata')

# 2D density plot -------------------------------------------------
png('./figures/2D age income gam.png', width=900, height=700, res=96)
par(mfrow = c(1, 1), mar = c(5, 5, 2, 2))
#first in 2D. quite nice actually
image2D(temp.data, clab = "Happiness",xlab="Age (scaled)", ylab="Income (scaled)", rasterImage = FALSE,
        cex.lab=1.5, cex.axis=1, cex.main=1.5, cex.sub=2, pt.cex = 2, colkey=list(cex.clab = 1.5))
### b&w
#image2D(temp.data, clab = "Happinness",xlab="Age", ylab="Income decile", rasterImage = FALSE,
#        cex.lab=1.5, cex.axis=1, cex.main=1.5, cex.sub=2, pt.cex = 2, col = ramp.col (col = c("white", "black"), n = 100, alpha = 0.8))
dev.off()

# 3D density plot, version 1 in color -------------------------------------------------
png('./figures/3D age income v1 gam.png', width=900*2, height=700*2, res=96)
par(mfrow = c(1, 1), mar = c(0, 0, 0, 0))
persp3D(z = temp.data, theta = 45, phi = 15, facets = T, curtain = TRUE, 
        shade=0.1, colkey = FALSE, xlab="Age (scaled)", ylab="Income (scaled)", zlab="Happiness", box=T, bty='b2',ticktype = "detailed")
dev.off()

# 3D density plot, version 2 in black and white -------------------------------------------------
png('./figures/3D age income v2 bw gam.png', width=900*2, height=700*2, res=96)
par(mfrow = c(1, 1), mar = c(0, 0, 0, 0))
persp3D(z = temp.data, theta = 45, phi = 10, facets = T, curtain = TRUE, 
        shade=0.6, col = ramp.col(c("white", "black")), border = "grey", colkey = FALSE, xlab="Age (scaled)", ylab="Income (scaled)", zlab="Happiness")

dev.off()

# The end of part 3 of 6