###### This file replicates the analysis of happiness, age and income published in 
###### Toshkov (2021) 'The Relationship Between Age and Happiness Varies by Income', Journal of Happiness Studies
###### Part 4 of 6 (Density plots over time). Last updated 1 September 2021

source ('ggplot2 graphical settings.R')
# Make a loop over all survey waves ---------------------------------------
years = seq(2002,2018,2)

filename = paste0('./figures/2D age income all waves.png')
png(filename = filename, width=1050, height=1500, res=96)
par(mfrow = c(4, 2), mar = c(5, 5, 3, 3))

for (w in 1:8) {
dat <- import_rounds(w, format = 'spss')
dat <- dat %>% 
  rename_all(recode, hinctnt = "hinctnta") %>%
  mutate (
    income = hinctnta, 
    country = factor(countrycode (cntry, origin='iso2c', destination = 'country.name')),
    feel.happy = as.numeric(happy),
    income.na = ifelse(is.na(income)==TRUE, 1, 0),
    income.f = factor (income),
    income.cat = ifelse(income < 3, 'Income deciles 1 and 2',ifelse(income > 8, 'Income deciles 9 and 10', 'Income deciles 3-7')),
    income.cat2 = ifelse(income < 3, 'Income deciles 1-2',ifelse(income > 7, 'Income deciles 8-10', NA)),
    age = agea,
    age.cat = cut (age, breaks = c(-Inf, 20,30,40,50,60,70,80,Inf), labels = c('15-20','21-30','31-40','41-50','51-60','61-70','71-80','80+')),
    age = ifelse(age>90, 90, ifelse(age<15, 15, age))
  ) 

#prepare data
x = dat$age
y = dat$income
z = dat$happy

elevation.df = data.frame(x,y,z)
elevation.loess = loess(z ~ x*y, data = elevation.df, degree=2, span=0.4)
xmin<- min(x, na.rm = T)
xmax<- max(x, na.rm = T)
ymin<- min(y, na.rm = T)
ymax<- max(y, na.rm = T)

elevation.fit = expand.grid(list(x = seq(xmin, xmax, 1), y = seq(ymin, ymax, 1)))
z = predict(elevation.loess, newdata = elevation.fit)
elevation.fit$Height = as.numeric(z)

#prepare matrix with x and y and rows/cols and z as the cell values
u1<-unique(elevation.fit$x)
u2<-unique(elevation.fit$y)
temp.data<-matrix(NA, nrow=length(u1), ncol=length(u2))

for (i in 1:length(u1)){
  for (j in 1:length(u2)){
    temp.data[i,j]<-elevation.fit$Height[elevation.fit$x==u1[i] & elevation.fit$y==u2[j]]
  }
}

ifelse (w %% 2 == 0, 
        image2D(temp.data, clab = ifelse (w==2, "Happiness", ''), xlab=ifelse(w>6, 'Age (scaled)', ''), ylab="", main= years[w],
                cex.lab=1.5, cex.axis=1, cex.main=1.5, cex.sub=2, pt.cex = 2, colkey=list(cex.clab = 1.5)),
        image2D(temp.data, main= years[w], xlab=ifelse(w>6, 'Age (scaled)', ''), ylab='Income (scaled)',
                cex.lab=1.5, cex.axis=1, cex.main=1.5, cex.sub=2, pt.cex = 2, colkey=FALSE))
}

dev.off()



# If needed to save 3D versions as well ---------------------------------
filename2 = paste0('./figures/3D age income v1 wave ', w,'.png')
png(filename2, width=900*2, height=700*2, res=96)
par(mfrow = c(1, 1), mar = c(0, 0, 0, 0))
persp3D(z = temp.data, theta = 45, phi = 15, facets = T, curtain = TRUE, 
        shade=0.1, colkey = FALSE, xlab="Age (scaled)", ylab="Income (scaled)", zlab="Happiness", box=T, bty='b2',ticktype = "detailed")
dev.off()

filename3 = paste0('./figures/3D age income v2 bw wave ', w,'.png')
png(filename3, width=900*2, height=700*2, res=96)
par(mfrow = c(1, 1), mar = c(0, 0, 0, 0))
persp3D(z = temp.data, theta = 45, phi = 10, facets = T, curtain = TRUE, 
        shade=0.6, col = ramp.col(c("white", "black")), border = "grey", colkey = FALSE, xlab="Age (scaled)", ylab="Income (scaled)", zlab="Happiness")

dev.off()

# The end of part 4 of 6
